#include "circuitmacrosmanager.h"

#include <KStandardDirs>
#include <KTar>

#include <QFile>
#include <QUrl>
#include <QHttp>
#include <QDebug>
#include <QMessageBox>
#include <QtNetwork>
#include <QProcess>
#include <QBuffer>
#include "httpdownloader.h"

CircuitMacrosManager::CircuitMacrosManager(): QObject()
{
}

bool CircuitMacrosManager::checkExistence() const
{
	QString cm_path = KStandardDirs::locateLocal("data", "cirkuit/circuit_macros/libcct.m4", true);

	if (QFile::exists(cm_path))
		return true;
	else
		return false;
}

void CircuitMacrosManager::downloadLatest()
{
	QString filename = KStandardDirs::locateLocal("data", "cirkuit/Circuit_macros.tar.gz", true);
	file = new QFile(filename);

	if (!file->open(QIODevice::WriteOnly))
	{
		 QMessageBox::information(0, tr("HTTP"),
								  tr("Unable to save the file %1: %2.")
								  .arg(filename).arg(file->errorString()));
		 delete file;
		 file = 0;
		 return;
	}
	
	dnldr = new HttpDownloader;
	MiniSpy spy(dnldr, SIGNAL(finished(const QUrl &, bool)));
	connect(dnldr, SIGNAL(done()), this, SLOT(done()));
	dnldr->addDownload(QUrl(QString("http://www.ece.uwaterloo.ca/~aplevich/Circuit_macros/Circuit_macros.tar.gz")), file);
}

void CircuitMacrosManager::done()
{
	qDebug("DONE");
	qDebug() << "Circuit macros downloaded";

	file->close();

	KTar tarfile(file->fileName());
	tarfile.open(QIODevice::ReadOnly);
	qDebug() << "TARNAME: " << tarfile.fileName();
	const KArchiveDirectory* root = tarfile.directory();
	qDebug() << "ENTRIES: " << root->entries();
	qDebug() << "IS_DIR: " << root->isDirectory();

	const KArchiveDirectory* mainDir = (KArchiveDirectory*) root->entry(root->entries().at(0));
	qDebug() << "ENTRIES: " << mainDir->entries();
	mainDir->copyTo(KStandardDirs::locateLocal("data", "cirkuit/circuit_macros/", true), true);

	configureCircuitMacros();
	installCctBatch();

	delete file;
	file = 0;

	emit(configured());
}

void CircuitMacrosManager::readmeDone()
{
	buffer->close();

	QString onlineVersion = findVersion(barray);
	qDebug() << "ONLINE version: " << onlineVersion;

	if (onlineVersion > installedVersion())
		emit newVersionAvailable(onlineVersion);

	delete buffer;
	buffer = 0;
}

void CircuitMacrosManager::configureCircuitMacros()
{
	QString homelibFilename = KStandardDirs::locateLocal("data", "cirkuit/circuit_macros/homelib.txt", false);
	QString defineString = QString("`define(`HOMELIB_',`%1')')").arg(KStandardDirs::locateLocal("data", "cirkuit/circuit_macros/", false));

	QFile homelibFile(homelibFilename);
	if (!homelibFile.open(QIODevice::WriteOnly | QIODevice::Text))
		 return;

	QTextStream out(&homelibFile);
	out << defineString << "\n";

	homelibFile.close();

	QStringList args;
	args << "homelib" << "psdefault";
	QProcess::startDetached("make", args, KStandardDirs::locateLocal("data", "cirkuit/circuit_macros/", false));

	QFile::remove(KStandardDirs::locateLocal("data", "cirkuit/Circuit_macros.tar.gz", false));

	qDebug() << "Circuit macros configured";
}

QString CircuitMacrosManager::installedVersion() const
{
	if (!checkExistence())
		return "";

	QString filename = KStandardDirs::locateLocal("data", "cirkuit/circuit_macros/README", false);

	QFile file(filename);
	if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
		return "";

	return findVersion(file.readAll());
}

void CircuitMacrosManager::checkOnlineVersion()
{
	barray.clear();
	buffer = new QBuffer(&barray);
	buffer->open(QIODevice::WriteOnly);

	dnldr = new HttpDownloader;
	connect(dnldr, SIGNAL(done()), this, SLOT(readmeDone()));
	dnldr->addDownload(QUrl(QString("http://www.ece.uwaterloo.ca/~aplevich/Circuit_macros/README")), buffer);
}

QString CircuitMacrosManager::findVersion(const QByteArray& byteArray) const
{
	bool versionStringFound = false;

	QRegExp rx("Version (\\d\\.\\d+)");
	QString version = "";

	QString content(byteArray);

	if (content.contains(rx))
	{
		versionStringFound = true;
		version = rx.cap(1);
	}

	return version;
}

void CircuitMacrosManager::installCctBatch()
{
	QString filename = KStandardDirs::locateLocal("data", "cirkuit/build/cctdpic", true);

	QFile file(filename);
	if (!file.open(QFile::WriteOnly | QFile::Text))
		return;

	QTextStream out(&file);

	out << "#!/bin/bash" << endl;
	out << "TMPNAME=tempA1R2C3W4i5e6r6d7" << endl;
	out << "CIRCUIT_MACROS_PATH=" << KStandardDirs::locateLocal("data", "cirkuit/circuit_macros") << endl << endl;

	out << "function mkltx()\n"
"{\n"
"cat > $TMPNAME.tex << EOF\n"
"\\documentclass[a4paper]{article}\n"
"% Get max paper size (empty except for one figure!)\n"
"% Clear all textual allowances other than the main canvas\n"
"\\setlength{\\oddsidemargin}{0pt}\n"
"\\setlength{\\topmargin}{0pt}\n"
"\\setlength{\\headheight}{0pt}\n"
"\\setlength{\\headsep}{0pt}\n"
"\\setlength{\\marginparsep}{0pt}\n"
"\\setlength{\\marginparwidth}{0pt}\n"
"\\setlength{\\footskip}{0pt}\n"
"% Top Left is 5mm,5mm Not 1inch,1inch\n"
"\\addtolength{\\oddsidemargin}{-20mm}\n"
"\\addtolength{\\topmargin}{-20mm}\n"
"% A4 is 210 x 297 mm. Make bottom right 5,5 remaining!\n"
"\\setlength{\\textwidth}{205mm}\n"
"\\setlength{\\textheight}{292mm}\n"
"\n"
"\\usepackage{rotating} % For rotated text in figs.\n"
"\\usepackage{amsmath}\n"
"\\usepackage{pstricks}\n"
"\\usepackage{boxdims}\n"
"\\newbox\\graph\n"
"\\thispagestyle{empty}\n"
"\\begin{document}\n"
"  {\\vspace{2ex}\\input{$TMPNAME.cct}\n"
"  \\box\\graph\n"
"  }\n"
"\\end{document}\n"
"EOF\n"
"return\n"
"}\n\n"
;

	out << "filpath=`dirname $1`\n"
"fn=$filpath/`basename $1 .m4`\n"
"if [ -e $fn.m4 ]\n"
"then\n"
"m4 $CIRCUIT_MACROS_PATH/pstricks.m4 $CIRCUIT_MACROS_PATH/libcct.m4 $fn.m4 | dpic -p > $TMPNAME.cct\n"
"if [ ! -e $TMPNAME.tex ]\n"
"then mkltx\n"
"fi\n"
"cp $TMPNAME.cct $fn.pst\n"
"latex $TMPNAME.tex -output-format=dvi\n"
"dvips -O 1in,1in -Pcmz -Pamz -o $TMPNAME.ps $TMPNAME\n"
"cp $TMPNAME.ps $fn.ps\n"
"gs -sDEVICE=ppmraw -r200 -sOutputFile=- -sNOPAUSE -q $TMPNAME.ps \\\n"
"-c showpage -c quit | pnmcrop| pnmmargin -white 10 | pnmalias \\\n"
"| pnmtopng >$fn.png\n"
"\n"
"# Fix bounding box\n"
"ps2eps -f $fn.ps\n"
"epstopdf $fn.eps\n"
"\n"
"rm -f $TMPNAME.*\n"
"else\n"
"echo \"$fn.m4 does not exist!\"\n"
"fi\n"
;

	file.close();

	file.setPermissions(QFile::ExeOwner | QFile::ReadOwner | QFile::WriteOwner);
}
